@@ -99,6 +99,11 @@ static iThreadResult run_SocketThread_(iThread *thread) {
     iSocketThread *d = (iAny *) thread;
     iMutex *smx = &d->socket->mutex;
     iBlock *inbuf = collect_Block(new_Block(0x20000));
+    iGuardMutex(smx, {
+        /* Connection has been formed. */
+        delete_Pipe(d->socket->stopConnect);
+        d->socket->stopConnect = NULL;
+    });
     while (value_Atomic(&d->mode) == run_SocketThreadMode) {
         if (bytesToSend_Socket(d->socket) > 0) {
             /* Make sure we won't block on select() when there's still data to send. */
@@ -248,7 +253,7 @@ static void init_Socket_(iSocket *d) {
     openEmpty_Buffer(d->input);
     d->fd = -1;
     d->address = NULL;
-    d->stopConnect = NULL;
+    d->stopConnect = new_Pipe(); /* used for aborting select() on user action */
     d->connecting = NULL;
     d->thread = NULL;
     init_Condition(&d->allSent);
@@ -280,6 +285,7 @@ void deinit_Socket(iSocket *d) {
 }
 
 static void startThread_Socket_(iSocket *d) {
+    iAssert(d->thread == NULL);
     d->thread = new_SocketThread(d, run_SocketThreadMode);
     start_SocketThread(d->thread);
 }
@@ -375,6 +381,7 @@ static iThreadResult connectAsync_Socket_(iThread *thd) {
                            strerror(errno));
                     continue;
                 }
+                iAssert(d->stopConnect != NULL);
                 const int stopFd = output_Pipe(d->stopConnect);
                 fd_set stopSet;
                 fd_set connSet;
@@ -425,8 +432,6 @@ static iThreadResult connectAsync_Socket_(iThread *thd) {
             unlock_Mutex(&d->mutex);
         }
     }
-    delete_Pipe(d->stopConnect);
-    d->stopConnect = NULL;
     if (rc) {
         int errNum;
         char *msg;
@@ -444,24 +449,25 @@ static iThreadResult connectAsync_Socket_(iThread *thd) {
 }
 
 static iBool open_Socket_(iSocket *d) {
-    /* The socket is assumed to be locked already. */
+    /* Note: The socket is assumed to be locked already. */
     if (isPending_Address(d->address)) {
-        /* TODO: Race -- what if Address finishes on this line? */
+        /* If Address finishes right now, addressLookedUp_Socket_() will block until the
+           mutex is available. When the address is resolved, the socket will be
+           opened via the callback. */
         setStatus_Socket_(d, connecting_SocketStatus);
-        return iTrue; // when address is resolved, the socket will be opened.
+        return iTrue;
     }
     else if (!isValid_Address(d->address)) {
         return iFalse;
     }
-    else {
+    else if (!d->connecting) {
         iAssert(d->fd == -1);
-        d->connecting = new_Thread(connectAsync_Socket_);
         setStatus_Socket_(d, connecting_SocketStatus);
+        d->connecting = new_Thread(connectAsync_Socket_);
         setUserData_Thread(d->connecting, d);
-        d->stopConnect = new_Pipe(); /* used for aborting select() on user action */
         start_Thread(d->connecting);
-        return iTrue;
     }
+    return iTrue; /* we're already connecting */
 }
 
 static void addressLookedUp_Socket_(iAny *any, const iAddress *address) {
