@@ -4,7 +4,7 @@
 
 import os, sys
 
-BUILD_LANGS = ['en', 'es', 'fi', 'ie', 'ru', 'sr', 'zh_Hans', 'zh_Hant']
+BUILD_LANGS = ['en', 'de', 'es', 'fi', 'fr', 'ie', 'ru', 'sr', 'zh_Hans', 'zh_Hant']
 MODE = 'compile'
 ESCAPES = {
     '\\': '\\',
