@@ -107,7 +107,8 @@ iRangecc        urlUser_String          (const iString *);
 iRangecc        urlRoot_String          (const iString *);
 const iString * absoluteUrl_String      (const iString *, const iString *urlMaybeRelative);
 iBool           isLikelyUrl_String      (const iString *);
-void            punyEncodeUrlHost_String(iString *);
+void            punyEncodeDomain_Rangecc(iRangecc domain, iString *encoded_out);
+void            punyEncodeUrlHost_String(iString *absoluteUrl);
 void            stripDefaultUrlPort_String(iString *);
 const iString * urlFragmentStripped_String(const iString *);
 void            urlDecodePath_String    (iString *);
