@@ -53,9 +53,10 @@ static void load_Lang_(iLang *d, const char *id) {
     /* Load compiled language strings from an embedded blob. */
     iUnused(id);
     const iBlock *data = equal_CStr(id, "fi")      ? &blobFi_Embedded
+                       : equal_CStr(id, "fr")      ? &blobFr_Embedded
                        : equal_CStr(id, "ru")      ? &blobRu_Embedded
                        : equal_CStr(id, "es")      ? &blobEs_Embedded
-//                       : equal_CStr(id, "de")      ? &blobDe_Embedded
+                       : equal_CStr(id, "de")      ? &blobDe_Embedded
                        : equal_CStr(id, "ie")      ? &blobIe_Embedded
                        : equal_CStr(id, "sr")      ? &blobSr_Embedded
                        : equal_CStr(id, "zh_Hans") ? &blobZh_Hans_Embedded
