@@ -24,6 +24,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #include "color.h"
 #include "metrics.h"
 #include "embedded.h"
+#include "window.h"
 #include "app.h"
 
 #define STB_TRUETYPE_IMPLEMENTATION
@@ -71,7 +72,7 @@ struct Impl_Glyph {
     iHashNode node;
     int flags;
     uint32_t glyphIndex;
-    const iFont *font; /* may come from symbols/emoji */
+    iFont *font; /* may come from symbols/emoji */
     iRect rect[2]; /* zero and half pixel offset */
     iInt2 d[2];
     float advance; /* scaled */
@@ -180,6 +181,7 @@ static void deinit_Font(iFont *d) {
 }
 
 static uint32_t glyphIndex_Font_(iFont *d, iChar ch) {
+    /* TODO: Add a small cache of ~5 most recently found indices. */
     const size_t entry = ch - 32;
     if (entry < iElemCount(d->indexTable)) {
         if (d->indexTable[entry] == ~0u) {
@@ -704,6 +706,7 @@ void cacheTextGlyphs_Font_(iFont *d, const iRangecc text) {
     iArray *     rasters = NULL;
     SDL_Texture *oldTarget = NULL;
     iBool        isTargetChanged = iFalse;
+    iAssert(isExposed_Window(get_Window()));
     /* We'll flush the buffered rasters periodically until everything is cached. */
     while (chPos < text.end) {
         while (chPos < text.end) {
@@ -994,11 +997,11 @@ static iRect run_Font_(iFont *d, const iRunArgs *args) {
             if (ch == '\r') { /* color change */
                 iChar esc = nextChar_(&chPos, args->text.end);
                 int colorNum = args->color;
-                if (esc != 0x24) { /* ASCII Cancel */
+                if (esc == '\r') { /* Extended range. */
+                    esc = nextChar_(&chPos, args->text.end) + asciiExtended_ColorEscape;
                     colorNum = esc - asciiBase_ColorEscape;
                 }
-                else if (esc == '\r') { /* Extended range. */
-                    esc = nextChar_(&chPos, args->text.end) + asciiExtended_ColorEscape;
+                else if (esc != 0x24) { /* ASCII Cancel */
                     colorNum = esc - asciiBase_ColorEscape;
                 }
                 if (mode & draw_RunMode && ~mode & permanentColorFlag_RunMode) {
@@ -1019,7 +1022,7 @@ static iRect run_Font_(iFont *d, const iRunArgs *args) {
         int x1 = iMax(xpos, xposExtend);
         /* Which half of the pixel the glyph falls on? */
         const int hoff = enableHalfPixelGlyphs_Text ? (xpos - x1 > 0.5f ? 1 : 0) : 0;
-        if (!isRasterized_Glyph_(glyph, hoff)) {
+        if (mode & draw_RunMode && !isRasterized_Glyph_(glyph, hoff)) {
             /* Need to pause here and make sure all glyphs have been cached in the text. */
             cacheTextGlyphs_Font_(d, args->text);
             glyph = glyph_Font_(d, ch); /* cache may have been reset */
@@ -1030,6 +1033,8 @@ static iRect run_Font_(iFont *d, const iRunArgs *args) {
             if (args->continueFrom_out) {
                 if (lastWordEnd != args->text.start && ~mode & noWrapFlag_RunMode) {
                     *args->continueFrom_out = skipSpace_CStr(lastWordEnd);
+                    *args->continueFrom_out = iMin(*args->continueFrom_out,
+                                                   args->text.end);
                 }
                 else {
                     *args->continueFrom_out = currentPos; /* forced break */
@@ -1098,11 +1103,6 @@ static iRect run_Font_(iFont *d, const iRunArgs *args) {
         if (!isSpace_Char(ch)) {
             xposExtend += isEmoji ? glyph->advance : advance;
         }
-        xposExtend = iMax(xposExtend, xpos);
-        xposMax    = iMax(xposMax, xposExtend);
-        if (args->continueFrom_out && ((mode & noWrapFlag_RunMode) || isWrapBoundary_(prevCh, ch))) {
-            lastWordEnd = currentPos; /* mark word wrap position */
-        }
 #if defined (LAGRANGE_ENABLE_KERNING)
         /* Check the next character. */
         if (!isMonospaced && glyph->font == d) {
@@ -1110,10 +1110,24 @@ static iRect run_Font_(iFont *d, const iRunArgs *args) {
             const char *peek = chPos;
             const iChar next = nextChar_(&peek, args->text.end);
             if (enableKerning_Text && !d->manualKernOnly && next) {
-                xpos += d->xScale * stbtt_GetGlyphKernAdvance(&d->font, glyph->glyphIndex, next);
+                const uint32_t nextGlyphIndex = glyphIndex_Font_(glyph->font, next);
+                const int kern = stbtt_GetGlyphKernAdvance(
+                    &glyph->font->font, glyph->glyphIndex, nextGlyphIndex);
+                if (kern) {
+//                    printf("%lc(%u) -> %lc(%u): kern %d (%f)\n", ch, glyph->glyphIndex, next,
+//                           nextGlyphIndex,
+//                           kern, d->xScale * kern);
+                    xpos       += d->xScale * kern;
+                    xposExtend += d->xScale * kern;
+                }
             }
         }
 #endif
+        xposExtend = iMax(xposExtend, xpos);
+        xposMax    = iMax(xposMax, xposExtend);
+        if (args->continueFrom_out && ((mode & noWrapFlag_RunMode) || isWrapBoundary_(prevCh, ch))) {
+            lastWordEnd = currentPos; /* mark word wrap position */
+        }
         prevCh = ch;
         if (--maxLen == 0) {
             break;
@@ -1122,6 +1136,7 @@ static iRect run_Font_(iFont *d, const iRunArgs *args) {
     if (args->runAdvance_out) {
         *args->runAdvance_out = xposMax - orig.x;
     }
+    fflush(stdout);
     return bounds;
 }
 
