@@ -28,11 +28,15 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #include <the_Foundation/math.h>
 #include <SDL_timer.h>
 
+#if defined (iPlatformAppleMobile)
+#   include "../ios.h"
+#endif
+
 iDeclareType(Touch)
 iDeclareType(TouchState)
 iDeclareType(Momentum)
 
-#define numHistory_Touch_   3
+#define numHistory_Touch_   5
 #define lastIndex_Touch_    (numHistory_Touch_ - 1)
 
 enum iTouchEdge {
@@ -41,16 +45,27 @@ enum iTouchEdge {
     right_TouchEdge,
 };
 
+enum iTouchAxis {
+    none_TouchAxis,
+    x_TouchAxis,
+    y_TouchAxis
+};
+
 struct Impl_Touch {
     SDL_FingerID id;
     iWidget *affinity; /* widget on which the touch started */
+    iWidget *edgeDragging;
     iBool hasMoved;
+    iBool isTapBegun;
+    iBool isTouchDrag;
     iBool isTapAndHold;
     enum iTouchEdge edge;
     uint32_t startTime;
     iFloat3 startPos;
+    enum iTouchAxis axis;
     uint32_t posTime[numHistory_Touch_];
     iFloat3 pos[numHistory_Touch_];
+    size_t posCount;
     iFloat3 accum;
 };
 
@@ -59,6 +74,7 @@ iLocalDef void pushPos_Touch_(iTouch *d, const iFloat3 pos, uint32_t time) {
     memmove(d->pos + 1, d->pos, (numHistory_Touch_ - 1) * sizeof(d->pos[0]));
     d->posTime[0] = time;
     d->pos[0]     = pos;
+    d->posCount++;
 }
 
 struct Impl_Momentum {
@@ -95,12 +111,19 @@ static iTouch *find_TouchState_(iTouchState *d, SDL_FingerID id) {
     return NULL;
 }
 
-static const uint32_t longPressSpanMs_ = 425;
-static const int      tapRadiusPt_     = 15;
+static const uint32_t longPressSpanMs_ = 500;
+static const int      tapRadiusPt_     = 10;
+
+iLocalDef float distance_Touch_(const iTouch *d) {
+    return length_F3(sub_F3(d->pos[0], d->startPos));
+}
+
+static iBool isStationaryDistance_Touch_(const iTouch *d, int distance) {
+    return !d->hasMoved && distance_Touch_(d) < distance * get_Window()->pixelRatio;
+}
 
 static iBool isStationary_Touch_(const iTouch *d) {
-    return !d->hasMoved &&
-           length_F3(sub_F3(d->pos[0], d->startPos)) < tapRadiusPt_ * get_Window()->pixelRatio;
+    return isStationaryDistance_Touch_(d, tapRadiusPt_);
 }
 
 static void dispatchMotion_Touch_(iFloat3 pos, int buttonState) {
@@ -114,8 +137,9 @@ static void dispatchMotion_Touch_(iFloat3 pos, int buttonState) {
     });
 }
 
-static void dispatchClick_Touch_(const iTouch *d, int button) {
+static iBool dispatchClick_Touch_(const iTouch *d, int button) {
     const iFloat3 tapPos = d->pos[0];
+    iWindow *window = get_Window();
     SDL_MouseButtonEvent btn = {
         .type = SDL_MOUSEBUTTONDOWN,
         .button = button,
@@ -126,13 +150,16 @@ static void dispatchClick_Touch_(const iTouch *d, int button) {
         .x = x_F3(tapPos),
         .y = y_F3(tapPos)
     };
-    dispatchEvent_Widget(get_Window()->root, (SDL_Event *) &btn);
+    iBool wasUsed = dispatchEvent_Widget(window->root, (SDL_Event *) &btn);
     /* Immediately released, too. */
     btn.type = SDL_MOUSEBUTTONUP;
     btn.state = SDL_RELEASED;
     btn.timestamp = SDL_GetTicks();
-    dispatchEvent_Widget(get_Window()->root, (SDL_Event *) &btn);
-    dispatchMotion_Touch_(zero_F3(), 0);
+    dispatchEvent_Widget(window->root, (SDL_Event *) &btn);
+    if (!wasUsed && button == SDL_BUTTON_RIGHT) {
+        postContextClick_Window(window, &btn);
+    }
+    return wasUsed;
 }
 
 static void clearWidgetMomentum_TouchState_(iTouchState *d, iWidget *widget) {
@@ -153,15 +180,25 @@ static void update_TouchState_(void *ptr) {
         iTouch *touch = i.value;
         /* Holding a touch will reset previous momentum for this widget. */
         if (isStationary_Touch_(touch)) {
-            if (nowTime - touch->startTime > 25) {
+            const int elapsed = nowTime - touch->startTime;
+            if (elapsed > 25) {
                 clearWidgetMomentum_TouchState_(d, touch->affinity);
             }
+            if (elapsed > 50 && !touch->isTapBegun) {
+                /* Looks like a possible tap. */
+                dispatchMotion_Touch_(touch->pos[0], 0);
+                touch->isTapBegun = iTrue;
+            }
             if (!touch->isTapAndHold && nowTime - touch->startTime >= longPressSpanMs_ &&
                 touch->affinity) {
                 dispatchClick_Touch_(touch, SDL_BUTTON_RIGHT);
                 touch->isTapAndHold = iTrue;
                 touch->hasMoved = iFalse;
                 touch->startPos = touch->pos[0];
+#if defined (iPlatformAppleMobile)
+                playHapticEffect_iOS(tap_HapticEffect);
+#endif
+                dispatchMotion_Touch_(init_F3(-100, -100, 0), 0);
             }
         }
     }
@@ -178,22 +215,27 @@ static void update_TouchState_(void *ptr) {
         iForEach(Array, m, d->moms) {
             if (numSteps == 0) break;
             iMomentum *mom = m.value;
+            if (!mom->affinity) {
+                remove_ArrayIterator(&m);
+                continue;
+            }
             for (int step = 0; step < numSteps; step++) {
                 mulvf_F3(&mom->velocity, momFriction);
                 addv_F3(&mom->accum, mulf_F3(mom->velocity, stepDurationMs / 1000.0f));
             }
-            const iInt2 points = initF3_I2(mom->accum);
-            if (points.x || points.y) {
-                subv_F3(&mom->accum, initI2_F3(points));
+            const iInt2 pixels = initF3_I2(mom->accum);
+            if (pixels.x || pixels.y) {
+                subv_F3(&mom->accum, initI2_F3(pixels));
                 dispatchEvent_Widget(mom->affinity, (SDL_Event *) &(SDL_MouseWheelEvent){
                     .type = SDL_MOUSEWHEEL,
                     .timestamp = nowTime,
-                    .which = 0, /* means "precise scrolling" in DocumentWidget */
-                    .x = points.x,
-                    .y = points.y
+                    .x = pixels.x,
+                    .y = pixels.y,
+                    .direction = perPixel_MouseWheelFlag
                 });
             }
             if (length_F3(mom->velocity) < minSpeed) {
+                setHover_Widget(NULL);
                 remove_ArrayIterator(&m);
             }
         }
@@ -220,6 +262,19 @@ void widgetDestroyed_Touch(iWidget *widget) {
     }
 }
 
+static void dispatchButtonDown_Touch_(iFloat3 pos) {
+    dispatchEvent_Widget(get_Window()->root, (SDL_Event *) &(SDL_MouseButtonEvent){
+        .type = SDL_MOUSEBUTTONDOWN,
+        .timestamp = SDL_GetTicks(),
+        .clicks = 1,
+        .state = SDL_PRESSED,
+        .which = SDL_TOUCH_MOUSEID,
+        .button = SDL_BUTTON_LEFT,
+        .x = x_F3(pos),
+        .y = y_F3(pos)
+    });
+}
+
 static void dispatchButtonUp_Touch_(iFloat3 pos) {
     dispatchEvent_Widget(get_Window()->root, (SDL_Event *) &(SDL_MouseButtonEvent){
         .type = SDL_MOUSEBUTTONUP,
@@ -233,6 +288,28 @@ static void dispatchButtonUp_Touch_(iFloat3 pos) {
     });
 }
 
+static iWidget *findOverflowScrollable_Widget_(iWidget *d) {
+    const iInt2 rootSize = rootSize_Window(get_Window());
+    for (iWidget *w = d; w; w = parent_Widget(w)) {
+        if (flags_Widget(w) & overflowScrollable_WidgetFlag) {
+            if (height_Widget(w) > rootSize.y && !hasVisibleChildOnTop_Widget(w)) {
+                return w;
+            }
+            return NULL;
+        }
+    }
+    return NULL;
+}
+
+static iWidget *findSlidePanel_Widget_(iWidget *d) {
+    for (iWidget *w = d; w; w = parent_Widget(w)) {
+        if (isVisible_Widget(w) && flags_Widget(w) & horizontalOffset_WidgetFlag) {
+            return w;
+        }
+    }
+    return NULL;
+}
+
 iBool processEvent_Touch(const SDL_Event *ev) {
     /* We only handle finger events here. */
     if (ev->type != SDL_FINGERDOWN && ev->type != SDL_FINGERMOTION && ev->type != SDL_FINGERUP) {
@@ -240,15 +317,20 @@ iBool processEvent_Touch(const SDL_Event *ev) {
     }
     iTouchState *d = touchState_();
     iWindow *window = get_Window();
+    if (!isFinished_Anim(&window->rootOffset)) {
+        return iFalse;
+    }
     const iInt2 rootSize = rootSize_Window(window);
     const SDL_TouchFingerEvent *fing = &ev->tfinger;
-    const iFloat3 pos = init_F3(fing->x * rootSize.x, fing->y * rootSize.y, 0); /* pixels */
+    const iFloat3 pos = add_F3(init_F3(fing->x * rootSize.x, fing->y * rootSize.y, 0), /* pixels */
+                               init_F3(0, -value_Anim(&window->rootOffset), 0));
     const uint32_t nowTime = SDL_GetTicks();
     if (ev->type == SDL_FINGERDOWN) {
         /* Register the new touch. */
         const float x = x_F3(pos);
         enum iTouchEdge edge = none_TouchEdge;
         const int edgeWidth = 30 * window->pixelRatio;
+        iWidget *dragging = NULL;
         if (x < edgeWidth) {
             edge = left_TouchEdge;
         }
@@ -256,64 +338,123 @@ iBool processEvent_Touch(const SDL_Event *ev) {
             edge = right_TouchEdge;
         }
         iWidget *aff = hitChild_Widget(window->root, init_I2(iRound(x), iRound(y_F3(pos))));
+        if (edge == left_TouchEdge) {
+            dragging = findSlidePanel_Widget_(aff);
+            if (dragging) {
+                setFlags_Widget(dragging, dragged_WidgetFlag, iTrue);
+            }
+        }
         /* TODO: We must retain a reference to the affinity widget, or otherwise it might
            be destroyed during the gesture. */
-//        printf("aff:%p (%s)\n", aff, aff ? class_Widget(aff)->name : "-");
-        if (flags_Widget(aff) & touchDrag_WidgetFlag) {
-            dispatchEvent_Widget(window->root, (SDL_Event *) &(SDL_MouseButtonEvent){
-                .type = SDL_MOUSEBUTTONDOWN,
-                .timestamp = fing->timestamp,
-                .clicks = 1,
-                .state = SDL_PRESSED,
-                .which = SDL_TOUCH_MOUSEID,
-                .button = SDL_BUTTON_LEFT,
-                .x = x_F3(pos),
-                .y = y_F3(pos)
-            });
-            edge = none_TouchEdge;
-        }
-        pushBack_Array(d->touches, &(iTouch){
+//        printf("aff:[%p] %s:'%s'\n", aff, aff ? class_Widget(aff)->name : "-",
+//               cstr_String(id_Widget(aff)));
+//        printf("drg:[%p] %s:'%s'\n", dragging, dragging ? class_Widget(dragging)->name : "-",
+//               cstr_String(id_Widget(dragging)));
+        iTouch newTouch = {
             .id = fing->fingerId,
             .affinity = aff,
-            .hasMoved = (flags_Widget(aff) & touchDrag_WidgetFlag) != 0,
+            .edgeDragging = dragging,
+//            .hasMoved = (flags_Widget(aff) & touchDrag_WidgetFlag) != 0,
             .edge = edge,
             .startTime = nowTime,
             .startPos = pos,
-            .pos = pos
-        });
-        /* Some widgets rely on hover state. */
-        dispatchMotion_Touch_(pos, 0);
+        };
+        pushPos_Touch_(&newTouch, pos, fing->timestamp);
+        pushBack_Array(d->touches, &newTouch);
+        /* Some widgets rely on hover state for scrolling. */
+        if (flags_Widget(aff) & hover_WidgetFlag && ~flags_Widget(aff) & touchDrag_WidgetFlag) {
+            setHover_Widget(aff);
+        }
         addTicker_App(update_TouchState_, d);
     }
     else if (ev->type == SDL_FINGERMOTION) {
         iTouch *touch = find_TouchState_(d, fing->fingerId);
         if (touch && touch->affinity) {
-            if (touch->isTapAndHold) {
-                pushPos_Touch_(touch, pos, fing->timestamp);
-                if (!touch->hasMoved && !isStationary_Touch_(touch)) {
-                    touch->hasMoved = iTrue;
-                }
-                dispatchMotion_Touch_(pos, 0);
+            if (touch->isTouchDrag) {
+                dispatchMotion_Touch_(pos, SDL_BUTTON_LMASK);
                 return iTrue;
             }
-            if (flags_Widget(touch->affinity) & touchDrag_WidgetFlag) {
-                dispatchMotion_Touch_(pos, SDL_BUTTON_LMASK);
+            if (touch->isTapAndHold) {
+                pushPos_Touch_(touch, pos, fing->timestamp);
+                if (!touch->hasMoved && !isStationaryDistance_Touch_(touch, tapRadiusPt_ * 3)) {
+                    touch->hasMoved = iTrue;
+                }
+                if (touch->hasMoved) {
+                    dispatchMotion_Touch_(pos, 0);
+                }
                 return iTrue;
             }
             /* Update touch position. */
             pushPos_Touch_(touch, pos, nowTime);
+            if (!touch->isTouchDrag && !isStationary_Touch_(touch) &&
+                flags_Widget(touch->affinity) & touchDrag_WidgetFlag) {
+                touch->hasMoved = iTrue;
+                touch->isTouchDrag = iTrue;
+                touch->edge = none_TouchEdge;
+                pushPos_Touch_(touch, pos, fing->timestamp);
+                dispatchMotion_Touch_(touch->startPos, 0);
+                dispatchEvent_Widget(window->root, (SDL_Event *) &(SDL_MouseButtonEvent){
+                    .type = SDL_MOUSEBUTTONDOWN,
+                    .timestamp = fing->timestamp,
+                    .clicks = 1,
+                    .state = SDL_PRESSED,
+                    .which = SDL_TOUCH_MOUSEID,
+                    .button = SDL_BUTTON_LEFT,
+                    .x = x_F3(touch->startPos),
+                    .y = y_F3(touch->startPos)
+                });
+                dispatchMotion_Touch_(pos, SDL_BUTTON_LMASK);
+                return iTrue;
+            }
             const iFloat3 amount = mul_F3(init_F3(fing->dx, fing->dy, 0),
                                           init_F3(rootSize.x, rootSize.y, 0));
             addv_F3(&touch->accum, amount);
             iInt2 pixels = initF3_I2(touch->accum);
             /* We're reporting scrolling as full points, so keep track of the precise distance. */
             subv_F3(&touch->accum, initI2_F3(pixels));
-            if (!touch->hasMoved && !isStationary_Touch_(touch)) {
-                touch->hasMoved = iTrue;
+            if (!touch->hasMoved) {
+                if (!isStationary_Touch_(touch)) {
+                    touch->hasMoved = iTrue;
+                    /* The first FINGERMOTION seems to be larger than the subsequence ones.
+                       Maybe SDL does its own stationary threshold? We'll counter by reducing
+                       the first one. */
+                    divvf_F3(&touch->accum, 6);
+                    divfv_I2(&pixels, 6);
+                    /* Allow scrolling a scrollable widget. */
+                    iWidget *flow = findOverflowScrollable_Widget_(touch->affinity);
+                    if (flow) {
+                        touch->affinity = flow;
+                    }
+                }
+                else {
+                    touch->accum = zero_F3();
+                    pixels       = zero_I2();
+                }
+            }
+            else if (!touch->axis &&
+                     distance_Touch_(touch) > tapRadiusPt_ * 3 * window->pixelRatio) {
+                /* Lock swipe direction to an axis. */
+                if (iAbs(x_F3(touch->startPos) - x_F3(pos)) >
+                    iAbs(y_F3(touch->startPos) - y_F3(pos)) * 1.5f) {
+                    touch->axis = x_TouchAxis;
+                }
+                else {
+                    touch->axis = y_TouchAxis;
+                }
             }
             /* Edge swipe aborted? */
-            if (touch->edge == left_TouchEdge && fing->dx < 0) {
-                touch->edge = none_TouchEdge;
+            if (touch->edge == left_TouchEdge) {
+                if (fing->dx < 0) {
+                    touch->edge = none_TouchEdge;
+                    if (touch->edgeDragging) {
+                        setFlags_Widget(touch->edgeDragging, dragged_WidgetFlag, iFalse);
+                        setVisualOffset_Widget(touch->edgeDragging, 0, 200, easeOut_AnimFlag);
+                        touch->edgeDragging = NULL;
+                    }
+                }
+                else if (touch->edgeDragging) {
+                    setVisualOffset_Widget(touch->edgeDragging, x_F3(pos) - x_F3(touch->startPos), 10, 0);
+                }
             }
             if (touch->edge == right_TouchEdge && fing->dx > 0) {
                 touch->edge = none_TouchEdge;
@@ -321,20 +462,25 @@ iBool processEvent_Touch(const SDL_Event *ev) {
             if (touch->edge) {
                 pixels.y = 0;
             }
+            if (touch->axis == x_TouchAxis) {
+                pixels.y = 0;
+            }
+            if (touch->axis == y_TouchAxis) {
+                pixels.x = 0;
+            }
 //            printf("%p (%s) py: %i wy: %f acc: %f\n",
 //                   touch->affinity,
 //                   class_Widget(touch->affinity)->name,
 //                   pixels.y, y_F3(amount), y_F3(touch->accum));
             if (pixels.x || pixels.y) {
-                dispatchMotion_Touch_(pos, 0);
+                setFocus_Widget(NULL);
                 dispatchEvent_Widget(touch->affinity, (SDL_Event *) &(SDL_MouseWheelEvent){
                     .type = SDL_MOUSEWHEEL,
                     .timestamp = SDL_GetTicks(),
-                    .which = 0, /* means "precise scrolling" in DocumentWidget */
                     .x = pixels.x,
                     .y = pixels.y,
+                    .direction = perPixel_MouseWheelFlag
                 });
-                dispatchMotion_Touch_(zero_F3(), 0);
                 /* TODO: Keep increasing movement if the direction is the same. */
                 clearWidgetMomentum_TouchState_(d, touch->affinity);
             }
@@ -347,15 +493,23 @@ iBool processEvent_Touch(const SDL_Event *ev) {
             if (touch->id != fing->fingerId) {
                 continue;
             }
+            if (touch->edgeDragging) {
+                setFlags_Widget(touch->edgeDragging, dragged_WidgetFlag, iFalse);
+            }
             if (touch->isTapAndHold) {
                 if (!isStationary_Touch_(touch)) {
                     dispatchClick_Touch_(touch, SDL_BUTTON_LEFT);
                 }
+                setHover_Widget(NULL);
                 remove_ArrayIterator(&i);
                 continue;
             }
             if (flags_Widget(touch->affinity) & touchDrag_WidgetFlag) {
+                if (!touch->isTouchDrag) {
+                    dispatchButtonDown_Touch_(touch->startPos);
+                }
                 dispatchButtonUp_Touch_(pos);
+//                setHover_Widget(NULL);
                 remove_ArrayIterator(&i);
                 continue;
             }
@@ -367,26 +521,29 @@ iBool processEvent_Touch(const SDL_Event *ev) {
                 !isStationary_Touch_(touch)) {
                 dispatchClick_Touch_(touch, touch->edge == left_TouchEdge ? SDL_BUTTON_X1
                                                                           : SDL_BUTTON_X2);
+                setHover_Widget(NULL);
             }
             else {
-                const uint32_t elapsed = fing->timestamp - touch->posTime[lastIndex_Touch_];
+                const size_t lastIndex = iMin(touch->posCount - 1, lastIndex_Touch_);
+                const uint32_t elapsed = fing->timestamp - touch->posTime[lastIndex];
                 const float minVelocity = 400.0f;
-                if (elapsed < 75) {
-                    velocity = divf_F3(sub_F3(pos, touch->pos[lastIndex_Touch_]),
+                if (elapsed < 150) {
+                    velocity = divf_F3(sub_F3(pos, touch->pos[lastIndex]),
                                        (float) elapsed / 1000.0f);
-                    if (fabsf(x_F3(velocity)) < minVelocity) {
+                    if (touch->axis == y_TouchAxis || fabsf(x_F3(velocity)) < minVelocity) {
                         setX_F3(&velocity, 0.0f);
                     }
-                    if (fabsf(y_F3(velocity)) < minVelocity) {
+                    if (touch->axis == x_TouchAxis || fabsf(y_F3(velocity)) < minVelocity) {
                         setY_F3(&velocity, 0.0f);
                     }
                 }
-//                printf("elap:%ums vel:%f\n", elapsed, length_F3(velocity));
+                //printf("elap:%ums vel:%f\n", elapsed, length_F3(velocity));
                 pushPos_Touch_(touch, pos, nowTime);
                 /* If short and didn't move far, do a tap (left click). */
                 if (duration < longPressSpanMs_ && isStationary_Touch_(touch)) {
                     dispatchMotion_Touch_(pos, SDL_BUTTON_LMASK);
                     dispatchClick_Touch_(touch, SDL_BUTTON_LEFT);
+                    dispatchMotion_Touch_(init_F3(-100, -100, 0), 0); /* out of screen */
                 }
                 else if (length_F3(velocity) > 0.0f) {
     //                printf("vel:%f\n", length_F3(velocity));
@@ -400,10 +557,11 @@ iBool processEvent_Touch(const SDL_Event *ev) {
                         d->lastMomTime = nowTime;
                     }
                     pushBack_Array(d->moms, &mom);
-                    dispatchMotion_Touch_(touch->startPos, 0);
+                    //dispatchMotion_Touch_(touch->startPos, 0);
                 }
                 else {
                     dispatchButtonUp_Touch_(pos);
+                    setHover_Widget(NULL);
                 }
             }
             remove_ArrayIterator(&i);
