@@ -906,16 +906,7 @@ static iBool scrollOverflow_Widget_(iWidget *d, int delta) {
 }
 
 iBool processEvent_Widget(iWidget *d, const SDL_Event *ev) {
-    if (ev->type == SDL_KEYDOWN) {
-        if (ev->key.keysym.sym == SDLK_TAB) {
-            setFocus_Widget(findFocusable_Widget(focus_Widget(),
-                                                 ev->key.keysym.mod & KMOD_SHIFT
-                                                     ? backward_WidgetFocusDir
-                                                     : forward_WidgetFocusDir));
-            return iTrue;
-        }
-    }
-    else if (d->flags & commandOnClick_WidgetFlag &&
+    if (d->flags & commandOnClick_WidgetFlag &&
              (ev->type == SDL_MOUSEBUTTONDOWN || ev->type == SDL_MOUSEBUTTONUP) &&
              (mouseGrab_Widget() == d || contains_Widget(d, init_I2(ev->button.x, ev->button.y)))) {
         postCommand_Widget(d,
@@ -1475,34 +1466,6 @@ void refresh_Widget(const iAnyObject *d) {
     postRefresh_App();
 }
 
-/* Debug utility for inspecting widget trees. */
-#include "labelwidget.h"
-static void printTree_Widget_(const iWidget *d, int indent) {
-    for (int i = 0; i < indent; ++i) {
-        fwrite("    ", 4, 1, stdout);
-    }
-    printf("[%p] %s:\"%s\" ", d, class_Widget(d)->name, cstr_String(&d->id));
-    if (isInstance_Object(d, &Class_LabelWidget)) {
-        printf("(%s|%s) ",
-               cstr_String(text_LabelWidget((const iLabelWidget *) d)),
-               cstr_String(command_LabelWidget((const iLabelWidget *) d)));
-    }
-    printf("size:%dx%d {min:%dx%d} [%d..%d %d:%d] flags:%08llx%s%s%s%s%s\n",
-           d->rect.size.x, d->rect.size.y,
-           d->minSize.x, d->minSize.y,
-           d->padding[0], d->padding[2],
-           d->padding[1], d->padding[3],
-           (long long unsigned int) d->flags,
-           d->flags & expand_WidgetFlag ? " exp" : "",
-           d->flags & tight_WidgetFlag ? " tight" : "",
-           d->flags & fixedWidth_WidgetFlag ? " fixW" : "",
-           d->flags & fixedHeight_WidgetFlag ? " fixH" : "",
-           d->flags & resizeToParentWidth_WidgetFlag ? " rsPrnW" : "");
-    iConstForEach(ObjectList, i, d->children) {
-        printTree_Widget_(i.object, indent + 1);
-    }
-}
-
 void raise_Widget(iWidget *d) {
     iPtrArray *onTop = onTop_RootData_();
     if (d->flags & keepOnTop_WidgetFlag) {
@@ -1525,16 +1488,67 @@ iBool hasVisibleChildOnTop_Widget(const iWidget *parent) {
     return iFalse;
 }
 
-void printTree_Widget(const iWidget *d) {
-    if (!d) {
-        printf("[NULL]\n");
-        return;
-    }
-    printTree_Widget_(d, 0);
-}
-
 iBeginDefineClass(Widget)
     .processEvent = processEvent_Widget,
     .draw         = draw_Widget,
 iEndDefineClass(Widget)
 
+/*----------------------------------------------------------------------------------------------
+   Debug utilities for inspecting widget trees.
+*/
+
+#include "labelwidget.h"
+static void printInfo_Widget_(const iWidget *d) {
+    printf("[%p] %s:\"%s\" ", d, class_Widget(d)->name, cstr_String(&d->id));
+    if (isInstance_Object(d, &Class_LabelWidget)) {
+        printf("(%s|%s) ",
+               cstr_String(text_LabelWidget((const iLabelWidget *) d)),
+               cstr_String(command_LabelWidget((const iLabelWidget *) d)));
+    }
+    printf("size:%dx%d {min:%dx%d} [%d..%d %d:%d] flags:%08llx%s%s%s%s%s\n",
+           d->rect.size.x, d->rect.size.y,
+           d->minSize.x, d->minSize.y,
+           d->padding[0], d->padding[2],
+           d->padding[1], d->padding[3],
+           (long long unsigned int) d->flags,
+           d->flags & expand_WidgetFlag ? " exp" : "",
+           d->flags & tight_WidgetFlag ? " tight" : "",
+           d->flags & fixedWidth_WidgetFlag ? " fixW" : "",
+           d->flags & fixedHeight_WidgetFlag ? " fixH" : "",
+           d->flags & resizeToParentWidth_WidgetFlag ? " rsPrnW" : "");
+}
+
+static void printTree_Widget_(const iWidget *d, int indent) {
+    for (int i = 0; i < indent; ++i) {
+        fwrite("    ", 4, 1, stdout);
+    }
+    printInfo_Widget_(d);
+    iConstForEach(ObjectList, i, d->children) {
+        printTree_Widget_(i.object, indent + 1);
+    }
+}
+
+void printTree_Widget(const iWidget *d) {
+    if (!d) {
+        puts("[NULL]");
+        return;
+    }
+    printTree_Widget_(d, 0);
+}
+
+void identify_Widget(const iWidget *d) {
+    if (!d) {
+        puts("[NULL}");
+        return;
+    }
+    int indent = 0;
+    for (const iWidget *w = d; w; w = w->parent, indent++) {
+        if (indent > 0) {
+            for (int i = 0; i < indent; ++i) {
+                fwrite("  ", 2, 1, stdout);
+            }
+        }
+        printInfo_Widget_(w);
+    }
+    fflush(stdout);
+}
