@@ -37,6 +37,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #include "touch.h"
 #include "../app.h"
 #include "../visited.h"
+#include "../history.h"
 #include "../gmcerts.h"
 #include "../gmutil.h"
 #include "../visited.h"
@@ -192,7 +193,7 @@ static const iMenuItem navMenuItems_[] = {
     { "${menu.openlocation}", SDLK_l, KMOD_PRIMARY, "navigate.focus" },
     { "---", 0, 0, NULL },
     { download_Icon " " saveToDownloads_Label, SDLK_s, KMOD_PRIMARY, "document.save" },
-    { "${menu.copy.source}", SDLK_c, KMOD_PRIMARY, "copy" },
+    { "${menu.page.copysource}", SDLK_c, KMOD_PRIMARY, "copy" },
     { "---", 0, 0, NULL },
     { leftHalf_Icon " ${menu.sidebar.left}", SDLK_l, KMOD_PRIMARY | KMOD_SHIFT, "sidebar.toggle" },
     { rightHalf_Icon " ${menu.sidebar.right}", SDLK_p, KMOD_PRIMARY | KMOD_SHIFT, "sidebar2.toggle" },
@@ -465,6 +466,18 @@ static void updateNavBarIdentity_(iWidget *navBar) {
     setFlags_Widget(as_Widget(idItem), disabled_WidgetFlag, !ident);
 }
 
+static void updateNavDirButtons_(iWidget *navBar) {
+    const iHistory *history = history_DocumentWidget(document_App());
+    setFlags_Widget(findChild_Widget(navBar, "navbar.back"), disabled_WidgetFlag,
+                    atOldest_History(history));
+    setFlags_Widget(findChild_Widget(navBar, "navbar.forward"), disabled_WidgetFlag,
+                    atLatest_History(history));
+    setFlags_Widget(findWidget_App("toolbar.back"), disabled_WidgetFlag,
+                    atOldest_History(history));
+    setFlags_Widget(findWidget_App("toolbar.forward"), disabled_WidgetFlag,
+                    atLatest_History(history));
+}
+
 static const int loadAnimIntervalMs_ = 133;
 static int       loadAnimIndex_      = 0;
 
@@ -677,6 +690,7 @@ static iBool handleNavBarCommands_(iWidget *navBar, const char *cmd) {
                 checkLoadAnimation_Window_(get_Window());
                 dismissPortraitPhoneSidebars_Window(get_Window());
                 updateNavBarIdentity_(navBar);
+                updateNavDirButtons_(navBar);
                 /* Icon updates should be limited to automatically chosen icons if the user
                    is allowed to pick their own in the future. */
                 if (updateBookmarkIcon_Bookmarks(bookmarks_App(), urlStr,
@@ -909,7 +923,7 @@ static void setupUserInterface_Window(iWindow *d) {
     setId_Widget(div, "navdiv");
     addChild_Widget(d->root, iClob(div));
 
-#if defined (LAGRANGE_CUSTOM_FRAME)
+#if defined (LAGRANGE_ENABLE_CUSTOM_FRAME)
     /* Window title bar. */
     if (prefs_App()->customFrame) {
         setPadding1_Widget(div, 1);
@@ -1033,7 +1047,7 @@ static void setupUserInterface_Window(iWindow *d) {
                 setAlignVisually_LabelWidget(fprog, iTrue);
                 addChildFlags_Widget(rightEmbed,
                                      iClob(fprog),
-                                     collapse_WidgetFlag | hidden_WidgetFlag);
+                                     collapse_WidgetFlag | frameless_WidgetFlag | hidden_WidgetFlag);
             }
             /* Download progress indicator is also inside the input field, but hidden normally. */ {
                 iLabelWidget *progress = new_LabelWidget(uiTextCaution_ColorEscape "00.000 ${mb}", NULL);
@@ -1156,11 +1170,22 @@ static void setupUserInterface_Window(iWindow *d) {
                         resizeWidthOfChildren_WidgetFlag |
                         arrangeHeight_WidgetFlag | arrangeHorizontal_WidgetFlag, iTrue);
         setBackgroundColor_Widget(toolBar, tmBannerBackground_ColorId);
-        addChildFlags_Widget(toolBar, iClob(newLargeIcon_LabelWidget("\U0001f870", "navigate.back")), frameless_WidgetFlag);
-        addChildFlags_Widget(toolBar, iClob(newLargeIcon_LabelWidget("\U0001f872", "navigate.forward")), frameless_WidgetFlag);
-        setId_Widget(addChildFlags_Widget(toolBar, iClob(newLargeIcon_LabelWidget("\U0001f464", "toolbar.showident")), frameless_WidgetFlag), "toolbar.ident");
-        setId_Widget(addChildFlags_Widget(toolBar, iClob(newLargeIcon_LabelWidget("\U0001f588", "toolbar.showview arg:-1")),
-                             frameless_WidgetFlag | commandOnClick_WidgetFlag), "toolbar.view");
+        setId_Widget(addChildFlags_Widget(toolBar,
+                                          iClob(newLargeIcon_LabelWidget("\U0001f870", "navigate.back")),
+                                          frameless_WidgetFlag),
+                     "toolbar.back");
+        setId_Widget(addChildFlags_Widget(toolBar,
+                                          iClob(newLargeIcon_LabelWidget("\U0001f872", "navigate.forward")),
+                                          frameless_WidgetFlag),
+                     "toolbar.forward");
+        setId_Widget(addChildFlags_Widget(toolBar,
+                                          iClob(newLargeIcon_LabelWidget("\U0001f464", "toolbar.showident")),
+                                          frameless_WidgetFlag),
+                     "toolbar.ident");
+        setId_Widget(addChildFlags_Widget(toolBar,
+                                          iClob(newLargeIcon_LabelWidget("\U0001f588", "toolbar.showview arg:-1")),
+                                          frameless_WidgetFlag | commandOnClick_WidgetFlag),
+                     "toolbar.view");
         iLabelWidget *menuButton = makeMenuButton_LabelWidget("\U0001d362", phoneNavMenuItems_,
                                                               iElemCount(phoneNavMenuItems_));
         setFont_LabelWidget(menuButton, uiLabelLarge_FontId);
@@ -1289,6 +1314,23 @@ static float pixelRatio_Window_(const iWindow *d) {
 #endif
 
 static float displayScale_Window_(const iWindow *d) {
+    /* The environment variable LAGRANGE_OVERRIDE_DPI can be used to override the automatic
+       display DPI detection. If not set, or is an empty string, ignore it.
+       Note: the same value used for all displays. */
+    const char *LAGRANGE_OVERRIDE_DPI = getenv("LAGRANGE_OVERRIDE_DPI");
+    if (LAGRANGE_OVERRIDE_DPI && *LAGRANGE_OVERRIDE_DPI) {
+        /* If the user has set the env var, but it is not an int, atoi */
+        /* will return 0, in which case we just guess and return 96 DPI.  */
+        const int envDpi = atoi(LAGRANGE_OVERRIDE_DPI);
+        if (envDpi > 0) {
+            return ((float) envDpi) / baseDPI_Window;
+        }
+        fprintf(stderr, "[Window] WARNING: failed to parse LAGRANGE_OVERRIDE_DPI='%s', "
+                "ignoring it\n", LAGRANGE_OVERRIDE_DPI);
+        /* To avoid showing the warning multiple times, overwrite
+         LAGRANGE_OVERRIDE_DPI with the empty string. */
+        setenv("LAGRANGE_OVERRIDE_DPI", "", 1);
+    }
 #if defined (iPlatformApple)
     iUnused(d);
     /* Apple UI sizes are fixed and only scaled by pixel ratio. */
@@ -1316,7 +1358,7 @@ static void drawBlank_Window_(iWindow *d) {
     SDL_RenderPresent(d->render);
 }
 
-#if defined (LAGRANGE_CUSTOM_FRAME)
+#if defined (LAGRANGE_ENABLE_CUSTOM_FRAME)
 static SDL_HitTestResult hitTest_Window_(SDL_Window *win, const SDL_Point *pos, void *data) {
     iWindow *d = data;
     iAssert(d->win == win);
@@ -1369,7 +1411,7 @@ SDL_HitTestResult hitTest_Window(const iWindow *d, iInt2 pos) {
 
 iBool create_Window_(iWindow *d, iRect rect, uint32_t flags) {
     flags |= SDL_WINDOW_RESIZABLE | SDL_WINDOW_ALLOW_HIGHDPI | SDL_WINDOW_HIDDEN;
-#if defined (LAGRANGE_CUSTOM_FRAME)
+#if defined (LAGRANGE_ENABLE_CUSTOM_FRAME)
     if (prefs_App()->customFrame) {
         /* We are drawing a custom frame so hide the default one. */
         flags |= SDL_WINDOW_BORDERLESS;
@@ -1379,7 +1421,7 @@ iBool create_Window_(iWindow *d, iRect rect, uint32_t flags) {
             width_Rect(rect), height_Rect(rect), flags, &d->win, &d->render)) {
         return iFalse;
     }
-#if defined (LAGRANGE_CUSTOM_FRAME)
+#if defined (LAGRANGE_ENABLE_CUSTOM_FRAME)
     if (prefs_App()->customFrame) {
         /* Register a handler for window hit testing (drag, resize). */
         SDL_SetWindowHitTest(d->win, hitTest_Window_, d);
@@ -1498,7 +1540,7 @@ void init_Window(iWindow *d, iRect rect) {
         SDL_FreeSurface(surf);
     }
     d->appIcon = NULL;
-#if defined (LAGRANGE_CUSTOM_FRAME)
+#if defined (LAGRANGE_ENABLE_CUSTOM_FRAME)
     /* Load the app icon for drawing in the title bar. */
     if (prefs_App()->customFrame) {
         SDL_Surface *surf = loadImage_(&imageLagrange64_Embedded, appIconSize_());
@@ -1561,7 +1603,7 @@ static iBool isNormalPlacement_Window_(const iWindow *d) {
 }
 
 static iBool unsnap_Window_(iWindow *d, const iInt2 *newPos) {
-#if defined (LAGRANGE_CUSTOM_FRAME)
+#if defined (LAGRANGE_ENABLE_CUSTOM_FRAME)
     if (!prefs_App()->customFrame) {
         return iFalse;
     }
@@ -1651,7 +1693,7 @@ static iBool handleWindowEvent_Window_(iWindow *d, const SDL_WindowEvent *ev) {
                 d->isMinimized = iTrue;
                 return iFalse;
             }
-#if defined (LAGRANGE_CUSTOM_FRAME)
+#if defined (LAGRANGE_ENABLE_CUSTOM_FRAME)
             /* Set the snap position depending on where the mouse cursor is. */
             if (prefs_App()->customFrame) {
                 SDL_Rect usable;
@@ -1681,7 +1723,7 @@ static iBool handleWindowEvent_Window_(iWindow *d, const SDL_WindowEvent *ev) {
                     return iTrue;
                 }
             }
-#endif /* defined LAGRANGE_CUSTOM_FRAME */
+#endif /* defined LAGRANGE_ENABLE_CUSTOM_FRAME */
             //printf("MOVED: %d, %d\n", ev->data1, ev->data2); fflush(stdout);
             if (unsnap_Window_(d, &newPos)) {
                 return iTrue;
@@ -1771,7 +1813,7 @@ static void applyCursor_Window_(iWindow *d) {
 
 iBool processEvent_Window(iWindow *d, const SDL_Event *ev) {
     switch (ev->type) {
-#if defined (LAGRANGE_CUSTOM_FRAME)
+#if defined (LAGRANGE_ENABLE_CUSTOM_FRAME)
         case SDL_SYSWMEVENT: {
             /* We observe native Win32 messages for better user interaction with the
                window frame. Mouse clicks especially will not generate normal SDL
@@ -1924,25 +1966,27 @@ void draw_Window(iWindow *d) {
     }
     /* Draw widgets. */
     d->frameTime = SDL_GetTicks();
-    draw_Widget(d->root);
-#if defined (LAGRANGE_CUSTOM_FRAME)
-    /* App icon. */
-    const iWidget *appIcon = findChild_Widget(d->root, "winbar.icon");
-    if (isVisible_Widget(appIcon)) {
-        const int   size    = appIconSize_();
-        const iRect rect    = bounds_Widget(appIcon);
-        const iInt2 mid     = mid_Rect(rect);
-        const iBool isLight = isLight_ColorTheme(colorTheme_App());
-        iColor iconColor    = get_Color(gotFocus || isLight ? white_ColorId : uiAnnotation_ColorId);
-        SDL_SetTextureColorMod(d->appIcon, iconColor.r, iconColor.g, iconColor.b);
-        SDL_SetTextureAlphaMod(d->appIcon, gotFocus || !isLight ? 255 : 92);
-        SDL_RenderCopy(
-            d->render,
-            d->appIcon,
-            NULL,
-            &(SDL_Rect){ left_Rect(rect) + gap_UI * 1.25f, mid.y - size / 2, size, size });
-    }
+    if (isExposed_Window(d)) {
+        draw_Widget(d->root);
+#if defined (LAGRANGE_ENABLE_CUSTOM_FRAME)
+        /* App icon. */
+        const iWidget *appIcon = findChild_Widget(d->root, "winbar.icon");
+        if (isVisible_Widget(appIcon)) {
+            const int   size    = appIconSize_();
+            const iRect rect    = bounds_Widget(appIcon);
+            const iInt2 mid     = mid_Rect(rect);
+            const iBool isLight = isLight_ColorTheme(colorTheme_App());
+            iColor iconColor    = get_Color(gotFocus || isLight ? white_ColorId : uiAnnotation_ColorId);
+            SDL_SetTextureColorMod(d->appIcon, iconColor.r, iconColor.g, iconColor.b);
+            SDL_SetTextureAlphaMod(d->appIcon, gotFocus || !isLight ? 255 : 92);
+            SDL_RenderCopy(
+                d->render,
+                d->appIcon,
+                NULL,
+                &(SDL_Rect){ left_Rect(rect) + gap_UI * 1.25f, mid.y - size / 2, size, size });
+        }
 #endif
+    }
 #if 0
     /* Text cache debugging. */ {
         SDL_Texture *cache = glyphCache_Text();
@@ -2070,7 +2114,7 @@ void setSnap_Window(iWindow *d, int snapMode) {
         }
         return;
     }
-#if defined (LAGRANGE_CUSTOM_FRAME)
+#if defined (LAGRANGE_ENABLE_CUSTOM_FRAME)
     if (d->place.snap == snapMode) {
         return;
     }
@@ -2136,7 +2180,7 @@ void setSnap_Window(iWindow *d, int snapMode) {
         arrange_Widget(d->root);
         postRefresh_App();
     }
-#endif /* defined (LAGRANGE_CUSTOM_FRAME) */
+#endif /* defined (LAGRANGE_ENABLE_CUSTOM_FRAME) */
 }
 
 int snap_Window(const iWindow *d) {
